<?php

use CRM_Eventics_ExtensionUtil as E;

class CRM_Eventics_Utils_Helper {

  /**
   * Generate an 'ics' attachment.
   * Returns an array or null.
   */
  public static function generateIcsAttachment(int $event_id, int $participant_id) {
    $ics_enabled = Civi::settings()->get('eventics_default_send_ics');

    $eventics = \Civi\Api4\Eventics::get(FALSE)
      ->selectRowCount()
      ->addWhere('event_id', '=', $event_id)
      ->execute();

    // If eventics settings exist, check if the feature is enabled for this event
    if ($eventics->count()) {
      // This feels redundant, but when using selectRowCount, it really returns just the count?
      $eventics = \Civi\Api4\Eventics::get(FALSE)
        ->addWhere('event_id', '=', $event_id)
        ->execute()
        ->first();
      if (empty($eventics['ics_enabled'])) {
        return;
      }
    }
    else {
      // Otherwise, check the global default setting
      if (!Civi::settings()->get('eventics_default_send_ics')) {
        return;
      }
    }

    // Check if the contact opted-out of ics attachemnts (and if the setting is enabled)
    if (Civi::settings()->get('eventics_optout')) {
      if ($participant_id) {
        $optout = \Civi\Api4\Participant::get(FALSE)
          ->addSelect('contact_id.EventICS_Preferences.Opt_out')
          ->addWhere('id', '=', $participant_id)
          ->execute()
          ->first();
        if (!empty($optout['contact_id.EventICS_Preferences.Opt_out'])) {
          return;
        }
      }
    }

    // getCompleteInfo will default to events ending in the future
    // and that can be annoying when testing.
    $start = '19700101000000';
    $end = $type = 0;
    $info = CRM_Event_BAO_Event::getCompleteInfo($start, $type, $event_id, $end, FALSE);

    if (!empty($eventics['event_location'])) {
      $info[0]['location'] = $eventics['event_location'];
      // Core won't add event.location if is_show_location is false
      $info[0]['is_show_location'] = 1;
    }
    if (!empty($eventics['event_description'])) {
      $info[0]['description'] = $eventics['event_description'];
    }

    $template = CRM_Core_Smarty::singleton();
    $template->assign('events', $info);

    // Timezone handling based on CRM/Event/ICalendar.php
    // and allow an event-specific timezone, if set
    $timezones = $eventics['event_timezone'] ? [$eventics['event_timezone']] : [@date_default_timezone_get()];
    $template->assign('timezone', $timezones[0]);
    $info[0]['timezone'] = $timezones[0];

    $date_min = min(
      array_map(function ($event) {
        return strtotime($event['start_date']);
      }, $info)
    );
    $date_max = max(
      array_map(function ($event) {
        return strtotime($event['end_date'] ?? $event['start_date']);
      }, $info)
    );

    $template->assign('timezones', CRM_Utils_ICalendar::generate_timezones($timezones, $date_min, $date_max));
    $calendar = $template->fetch('CRM/Core/Calendar/ICal.tpl');
    $calendar = preg_replace('/(?<!\r)\n/', "\r\n", $calendar);

    $config = CRM_Core_Config::singleton();
    $moveTo = '';

    // For Outlook links we need to move the ICS file
    if ($outlookLink) {
      $moveTo = '[civicrm.files]/persist/ics/';
      if (!is_dir(Civi::paths()->getPath($moveTo))) {
        mkdir(Civi::paths()->getPath($moveTo));
      }
      $icsfile = tempnam(Civi::paths()->getPath($moveTo), 'ics');
      chmod($icsfile, 0644);
    }
    else {
      $icsfile = tempnam($config->customFileUploadDir, 'ics');
    }

    if ($icsfile !== FALSE) {
      rename($icsfile, $icsfile . '.ics');
      $icsfile .= '.ics';

      if (file_put_contents($icsfile, $calendar) !== FALSE) {
        $attachment = [
          'fullPath' => $icsfile,
          'mime_type' => 'text/calendar',
          'cleanName' => 'invitation_' . date('YmdHis') . '.ics',
        ];
        return $attachment;
      }
    }

    return NULL;
  }


  /**
   * Generated a Google Calendar Link to add event
   *
   * Based on https://zaclee.net/php/formatting-google-calendar-links-with-timezones-and-php-carbon
   */
  public static function generateGoogleCalendarLink($title, $begin, $end, $location, $details, $timezone) {
    $queryString = sprintf(
      'action=%s&text=%s&details=%s&location=%s&dates=%s/%s&ctz=%s',
      // action
      'TEMPLATE',
      // text
      urlencode($title),
      // details
      urlencode(strip_tags($details)),
      // location
      urlencode($location),
      // dates
      urlencode(CRM_Utils_Date::customFormat($begin, "%Y%m%dT%H%i%s")),
      urlencode(CRM_Utils_Date::customFormat($end, "%Y%m%dT%H%i%s")),
      // timezone
      urlencode($timezone),
    );

    return 'https://www.google.com/calendar/event?' . $queryString;
  }

  /**
   * Return a list of timezones
   * Copied from ext/standaloneusers/CRM/Standaloneusers/BAO/User.php
   */
  public static function getTimeZones(): array {
    $timeZones = [];
    foreach (\DateTimeZone::listIdentifiers() as $timezoneId) {
      $timeZones[$timezoneId] = $timezoneId;
    }
    return $timeZones;
  }

  /**
   * Return the default site timezone
   *
   * There does not seem to be a CiviCRM core function for this,
   * only getTimeZoneString for the user's timezone.
   */
   public static function getSiteDefaultTimezone() {
     if (CIVICRM_UF === 'Standalone') {
       // @todo Not what we want, because this will be the timezone of the user
       return date_default_timezone_get();
     }
     elseif (CIVICRM_UF === 'Backdrop') {
       return config_get('system.date', 'timezone.default');
     }
     elseif (CIVICRM_UF === 'Drupal8') {
       return \Drupal::config('system.date')->get('timezone.default');
     }
     elseif (CIVICRM_UF === 'Drupal') {
       return variable_get('date_default_timezone', date_default_timezone_get());
     }
     elseif (CIVICRM_UF === 'WordPress') {
       // @todo Not tested
       $timezone = get_option('timezone_string');
       if (!empty($timezone)) {
         return $timezone;
       }
       $offset = get_option('gmt_offset');
       $timezone = timezone_name_from_abbr('', $offset * 3600, 0);
       // Fallback if PHP can't resolve the abbreviation
       if ($timezone === false) {
         return date_default_timezone_get();
       }
       return $timezone;
     }
     throw new Exception('Unknown CIVICRM_UF: ' . CIVICRM_UF);
   }

  /**
   * When the optout setting is changed, either create the field, or warn
   * admins about deletion.
   *
   * @param $old_value
   * @param $new_value
   */
  public static function onChangeOptoutSetting($old_value, $new_value) {
    if ($new_value != $old_value) {
      if ($new_value) {
        CRM_Core_Session::setStatus(E::ts("A custom group '%1' has been created.", [1 => 'EventICS Preferences']), '', 'success');
      }
      \CRM_Core_ManagedEntities::singleton()->reconcile(['eventics']);
    }
  }


}
